#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!

#  CPUGovernor (c) Andi Hasenkopf 2010 <andreas.hasenkopf@physik.uni-regensburg.de>

import screenlets
from screenlets.options import StringOption , BoolOption , IntOption , FileOption , DirectoryOption , ListOption , AccountOption , TimeOption , FontOption, ColorOption , ImageOption
from screenlets.options import create_option_from_node
from screenlets import DefaultMenuItem
import pango
#import gobject
import gtk
import subprocess
import os
import sys

class CPUGovernorScreenlet (screenlets.Screenlet):
	"""A simple example of how to create a Screenlet"""
	
	# default meta-info for Screenlets (should be removed and put into metainfo)
	__name__	= 'CPUGovernorScreenlet'
	__version__	= '0.1'
	__author__	= 'Andi Hasenkopf'
	__desc__	= __doc__	# set description to docstring of class
	
	# editable options (options that are editable through the UI)
	icon = 'cpu'
	backgr = 'back'
	acc = ('','')
	font = "Sans\ 10"
	font_color = (1,1,1, 0.8)
	board_color_0 = (.1,.1,.1, .2)
	board_color_1 = (.05,.05,.05, 0.8)
	shadow_color = (0,0,0, 0.6)
	highlight_color_0 = (1,0.7,0.7,0.8)
	highlight_color_1 = (0.7,1,0.7,0.8)
	show_details = False

	cpufreq_info = False
	cpufreq_set = False
	sudo = False
	cpufreq_governors = []
	cpufreq_freqs = []

	# constructor
	def __init__ (self, **keyword_args):
		#call super (width/height MUST match the size of graphics in the theme)
		screenlets.Screenlet.__init__(self, width=int(128*self.scale), height=int(128*self.scale), 
			uses_theme=True, **keyword_args)
		# set theme
		self.theme_name = "default"
		# add option group
		self.add_options_group('CPUGovernor', 'Font Config ...')

		# add editable option to the group
		self.add_option(FontOption('CPUGovernor','font', 
			self.font, 'Font', 
			'This font will be used.'))

		self.add_option(ColorOption('CPUGovernor','font_color', 
			self.font_color, 'Font color', 
			'This is the font\'s color'))

		self.add_option(ColorOption('CPUGovernor','board_color_0', 
			self.board_color_0, 'Board color #1', 
			'This is the board\'s color #1'))

		self.add_option(ColorOption('CPUGovernor','shadow_color', 
			self.shadow_color, 'Shadow color', 
			'This is the shadow color'))

		self.add_option(ColorOption('CPUGovernor','highlight_color_0', 
			self.highlight_color_0, 'Highlight color 1', 
			'This color is used for highlighting events'))

		self.test_cpufreq()
		self.test_sudo()

	def test_sudo(self):
		prog = subprocess.Popen(['whereis','gksudo'],stdout=subprocess.PIPE)
		value = prog.communicate()[0].split(':')
		if len(value[1]):
			value=value[1].split(' ')
			for prog in value:
				if os.access(prog,os.X_OK):
					self.sudo = prog
					break
			else:
				screenlets.show_message(self, 'This screenlet requieres the gksudo utility program!')
				sys.exit(1)
		return 0

	def test_cpufreq(self):
		prog = subprocess.Popen(['whereis','cpufreq-info'],stdout=subprocess.PIPE)
		value = prog.communicate()[0].split(':')
		if len(value[1]):
			value=value[1].split(' ')
			for prog in value:
				if os.access(prog,os.X_OK):
					self.cpufreq_info = prog
					break
				
		prog = subprocess.Popen(['whereis','cpufreq-set'],stdout=subprocess.PIPE)
		value = prog.communicate()[0].split(':')
		if len(value[1]):
			value=value[1].split(' ')
			for prog in value:
				if os.access(prog,os.X_OK):
					self.cpufreq_set = prog
					break
	
		if not (self.cpufreq_info and self.cpufreq_set):
			screenlets.show_message(self, 'This screenlet requieres the cpufreq utilities!')
			sys.exit(1)
		else:
			prog = subprocess.Popen([self.cpufreq_info, '-c0', '-g'],stdout=subprocess.PIPE)
			self.cpufreq_governors = prog.communicate()[0].split()
			try:
				prog = open('/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies','r')
				self.cpufreq_freqs = prog.readline().split()
				prog.close()
			except:
				pass
		return 0

	def get_cpu_state(self,n):
		prog = subprocess.Popen([self.cpufreq_info,'-c%s' % n,'-p'],stdout=subprocess.PIPE)
		return prog.communicate()[0].split()
		
	def draw_cpu_state(self,ctx,n):
		value = self.get_cpu_state(n)
		ctx.save()
		ctx.set_source_rgba(*self.font_color)
		self.draw_text(ctx, '<b>CPU%s</b>' % n, 0, 0, self.font , 12,self.width-32,pango.ALIGN_LEFT)
		ctx.translate(8,16)
		self.draw_text(ctx, value[2],0,0,self.font,8,self.width-32,pango.ALIGN_LEFT)
		ctx.translate(8,16)
		self.draw_text(ctx, '(%.1f - %.1f GHz)' % (int(value[0])/1e6,int(value[1])/1e6),0,0,self.font,6,self.width-32,pango.ALIGN_LEFT)
		ctx.restore()
		return 0

	def set_cpu_states(self):
		diag = gtk.Dialog(
			title="Set CPU Governor",
			parent=None,
			buttons=(gtk.STOCK_OK,True,gtk.STOCK_CANCEL,False)
		)
		table = gtk.Table(3,4)
		diag.vbox.add(table)
		
		table.attach(gtk.Label('CPU0'),1,2,0,1)
		table.attach(gtk.Label('CPU1'),2,3,0,1)
		table.attach(gtk.Label('Gov:'),0,1,1,2)
		table.attach(gtk.Label('Min. Freq.'),0,1,2,3)
		table.attach(gtk.Label('Max. Freq.'),0,1,3,4)

		gov0 = gtk.combo_box_new_text()
		gov1 = gtk.combo_box_new_text()
		min0 = gtk.combo_box_new_text()
		min1 = gtk.combo_box_new_text()
		max0 = gtk.combo_box_new_text()
		max1 = gtk.combo_box_new_text()
		
		for g in self.cpufreq_governors:
			gov0.append_text(g)
			gov1.append_text(g)
	
		for f in self.cpufreq_freqs:
			f = int(f)/1e6
			min0.append_text('%.3f GHz' % f)
			min1.append_text('%.3f GHz' % f)
			max0.append_text('%.3f GHz' % f)
			max1.append_text('%.3f GHz' % f)

		v = self.get_cpu_state(0)
		if v[2] in self.cpufreq_governors:
			gov0.set_active(self.cpufreq_governors.index(v[2]))
		if v[0] in self.cpufreq_freqs:
			min0.set_active(self.cpufreq_freqs.index(v[0]))
		if v[1] in self.cpufreq_freqs:
			max0.set_active(self.cpufreq_freqs.index(v[1]))

		v = self.get_cpu_state(1)
		if v[2] in self.cpufreq_governors:
			gov1.set_active(self.cpufreq_governors.index(v[2]))
		if v[0] in self.cpufreq_freqs:
			min1.set_active(self.cpufreq_freqs.index(v[0]))
		if v[1] in self.cpufreq_freqs:
			max1.set_active(self.cpufreq_freqs.index(v[1]))
	
		table.attach(gov0,1,2,1,2)
		table.attach(gov1,2,3,1,2)
		table.attach(min0,1,2,2,3)
		table.attach(min1,2,3,2,3)
		table.attach(max0,1,2,3,4)
		table.attach(max1,2,3,3,4)

		table.show_all()
		result = diag.run()
		if result:
			cmd = '%s "%s -c%s -g %s -d %s -u %s"' % (self.sudo,self.cpufreq_set,0,self.cpufreq_governors[gov0.get_active()],self.cpufreq_freqs[min0.get_active()],self.cpufreq_freqs[max0.get_active()])
			prog = os.popen(cmd)
			errors = prog.readlines()
			if len(errors):
				screenlets.show_message(self, 'CPU propertires couldn\'t be changed:\n %s' % '\n'.join(errors))

			cmd = '%s "%s -c%s -g %s -d %s -u %s"' % (self.sudo,self.cpufreq_set,1,self.cpufreq_governors[gov1.get_active()],self.cpufreq_freqs[min1.get_active()],self.cpufreq_freqs[max1.get_active()])
			prog = os.popen(cmd)
			errors = prog.readlines()
			if len(errors):
				screenlets.show_message(self, 'CPU propertires couldn\'t be changed:\n %s' % '\n'.join(errors))
		diag.destroy()

	def set_cpu_gov(self,n,governor):
		cmd = '%s "%s -c%s -g %s"' % (self.sudo,self.cpufreq_set,n,governor)
		prog = os.popen(cmd)
		errors = prog.readlines()
		if len(errors):
			screenlets.show_message(self, 'CPU propertires couldn\'t be changed:\n %s' % '\n'.join(errors))
		return 0

	# ONLY FOR TESTING!!!!!!!!!
	def init_options_from_metadata (self):
		"""Try to load metadata-file with options. The file has to be named
		like the Screenlet, with the extension ".xml" and needs to be placed
		in the Screenlet's personal directory. 
		NOTE: This function always uses the metadata-file relative to the 
			  Screenlet's location, not the ones in SCREENLETS_PATH!!!"""
		pass

	def on_after_set_atribute(self,name, value):
		"""Called after setting screenlet atributes"""
		pass

	def on_before_set_atribute(self,name, value):
		"""Called before setting screenlet atributes"""
		pass


	def on_create_drag_icon (self):
		"""Called when the screenlet's drag-icon is created. You can supply
		your own icon and mask by returning them as a 2-tuple."""
		return (None, None)

	def on_composite_changed(self):
		"""Called when composite state has changed"""
		pass

	def on_drag_begin (self, drag_context):
		"""Called when the Screenlet gets dragged."""
		pass
	
	def on_drag_enter (self, drag_context, x, y, timestamp):
		"""Called when something gets dragged into the Screenlets area."""
		pass
	
	def on_drag_leave (self, drag_context, timestamp):
		"""Called when something gets dragged out of the Screenlets area."""
		pass

	def on_drop (self, x, y, sel_data, timestamp):
		"""Called when a selection is dropped on this Screenlet."""
		return False
		
	def on_focus (self, event):
		"""Called when the Screenlet's window receives focus."""
		pass
	
	def on_hide (self):
		"""Called when the Screenlet gets hidden."""
		pass
	
	def on_init (self):
		"""Called when the Screenlet's options have been applied and the 
		screenlet finished its initialization. If you want to have your
		Screenlet do things on startup you should use this handler."""
		for g in self.cpufreq_governors:
			self.add_menuitem("ch_governor_%s" % g, 'Change to: %s' % g)
		# add default menu items
		self.add_default_menuitems()

	def on_key_down(self, keycode, keyvalue, event):
		"""Called when a keypress-event occured in Screenlet's window."""
		pass
	
	def on_load_theme (self):
		"""Called when the theme is reloaded (after loading, before redraw)."""
		pass
	
	def on_menuitem_select (self, id):
		"""Called when a menuitem is selected."""

		if id[:12] == 'ch_governor_':
			self.set_cpu_gov(0,id[12:])
			self.set_cpu_gov(1,id[12:])
		#if id == "update_now":
		#	self.update()
		pass
	
	def on_mouse_down (self, event):
		"""Called when a buttonpress-event occured in Screenlet's window. 
		Returning True causes the event to be not further propagated."""
		if event.button == 2:
			self.set_cpu_states()
		return False
	
	def on_mouse_enter (self, event):
		"""Called when the mouse enters the Screenlet's window."""
#		pass
		self.show_details = True
		self.redraw_canvas()
		
	def on_mouse_leave (self, event):
		"""Called when the mouse leaves the Screenlet's window."""
#		pass
		self.show_details = False
		self.redraw_canvas()

	def on_mouse_move(self, event):
		"""Called when the mouse moves in the Screenlet's window."""
		self.redraw_canvas()
		pass

	def on_mouse_up (self, event):
		"""Called when a buttonrelease-event occured in Screenlet's window. 
		Returning True causes the event to be not further propagated."""
		return False
	
	def on_quit (self):
		"""Callback for handling destroy-event. Perform your cleanup here!"""
		print 'Bye'
		return True
		
	def on_realize (self):
		""""Callback for handling the realize-event."""
		pass
	
	def on_scale (self):
		"""Called when Screenlet.scale is changed."""
		pass
	
	def on_scroll_up (self):
		"""Called when mousewheel is scrolled up (button4)."""
		pass

	def on_scroll_down (self):
		"""Called when mousewheel is scrolled down (button5)."""
		pass
	
	def on_show (self):
		"""Called when the Screenlet gets shown after being hidden."""
		pass
	
	def on_switch_widget_state (self, state):
		"""Called when the Screenlet enters/leaves "Widget"-state."""
		pass
	
	def on_unfocus (self, event):
		"""Called when the Screenlet's window loses focus."""
		pass
	
	def on_draw (self, ctx):
		# if theme is loaded
		if self.theme:
			ctx.scale(self.scale, self.scale)
			self.theme.render(ctx, self.icon)

			if self.show_details:
				ctx.set_source_rgba(*self.board_color_0)
				self.draw_rectangle_advanced (ctx, 16, 16, 96, 96, rounded_angles=(2,2,2,2), fill=True, border_size=0, border_color=self.board_color_0, shadow_size=0, shadow_color=self.shadow_color)

				ctx.translate(16,16)
				try:
					self.draw_cpu_state(ctx,0)
				except:
					pass
				ctx.translate(0,48)
				try:
					self.draw_cpu_state(ctx,1)
				except:
					pass

	def on_draw_shape (self, ctx):
		self.on_draw(ctx)
	
# If the program is run directly or passed as an argument to the python
# interpreter then create a Screenlet instance and show it
if __name__ == "__main__":
	# create new session
	import screenlets.session
	screenlets.session.create_session(CPUGovernorScreenlet)

